///////////////////////////////////////////////////////////////////////
//                                                                   //
//   Copyright 2012 David Alonso                                     //
//                                                                   //
//                                                                   //
// This file is part of CRIME.                                       //
//                                                                   //
// CRIME is free software: you can redistribute it and/or modify it  //
// under the terms of the GNU General Public License as published by //
// the Free Software Foundation, either version 3 of the License, or //
// (at your option) any later version.                               //
//                                                                   //
// CRIME is distributed in the hope that it will be useful, but      //
// WITHOUT ANY WARRANTY; without even the implied warranty of        //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU //
// General Public License for more details.                          //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with CRIME.  If not, see <http://www.gnu.org/licenses/>.    //
//                                                                   //
///////////////////////////////////////////////////////////////////////
#include "common_gh.h"

int main(int argc,char **argv)
{ 
  char fnameIn[256];
  if(argc!=2) {
    fprintf(stderr,"Usage: ./GetHI file_name\n");
    exit(0);
  }
  sprintf(fnameIn,"%s",argv[1]);

  setbuf(stdout,NULL);
  printf("\n");
  printf("|-------------------------------------------------|\n");
  printf("|                      GetHI                      |\n");
  printf("|-------------------------------------------------|\n\n");

  timer(4);

  read_run_params(fnameIn);

  if(signed_seed_rng<0) seed_rng=time(NULL);
  else seed_rng=(unsigned int)signed_seed_rng;
  printf("Seed : %u\n",seed_rng);

  flouble *dens_arr;
  flouble *vr_arr;
  flouble *T_maps;

  //Arrays for point sources
  int *nsources=NULL;
  flouble *T_maps_psources=NULL;

  //Create Gaussian density and radial velocity fields
  create_d_and_vr_fields(&seed_rng,&dens_arr,&vr_arr);

  //Poisson-sample for point sources, transform to HI mass and to Dz_RSD
  if(do_psources) {
    setup_psources();
    nsources=get_point_sources(dens_arr,&seed_rng);
  }
  get_HI(dens_arr,vr_arr);

  //Pixelize maps
  T_maps=mk_T_maps(&seed_rng,dens_arr,vr_arr);
  free(dens_arr);
  if(do_psources)
    T_maps_psources=mk_psources_maps(&seed_rng,nsources,vr_arr);
  free(vr_arr);
  if(do_psources)
    free(nsources);

  //Write temperature maps
  write_maps(fnameOut,T_maps,T_maps_psources);
  free(T_maps);
  if(do_psources)
    free(T_maps_psources);
  timer(5);

  printf("\n");
  printf("|-------------------------------------------------|\n\n");

  end_cosmo();

  return 0;
}
